<?php
/**
 * @package     CSVI
 * @subpackage  Form2Content
 *
 * @author      RolandD Cyber Produksi <contact@csvimproved.com>
 * @copyright   Copyright (C) 2006 - 2017 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://csvimproved.com
 */

defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;

/**
 * Form2Content Article import.
 *
 * @package     CSVI
 * @subpackage  Form2Content
 * @since       7.0
 */
class Com_Form2ContentModelImportF2carticle extends RantaiImportEngine
{
	/**
	 * Form2Content table.
	 *
	 * @var    Form2ContentTableF2carticle
	 * @since  7.0
	 */
	private $f2carticle = null;

	/**
	 * Joomla content table.
	 *
	 * @var    ContentTableContent
	 * @since  7.0
	 */
	private $contentTable = null;

	/**
	 * The Form2Content helper
	 *
	 * @var    Com_Form2ContentHelperCom_Form2Content
	 * @since  7.0
	 */
	protected $helper = null;

	/**
	 * The article form id
	 *
	 * @since  7.0
	 */
	protected $formId = null;

	/**
	 * The project field id
	 *
	 * @since  7.0
	 */
	protected $fieldId = null;

	/**
	 * List of available content fields
	 *
	 * @var    array
	 * @since  7.0
	 */
	private $contentFields = '';

	/**
	 * Run this before we start.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function onBeforeStart()
	{
		// Load the content type fields
		$this->loadContentFields();
	}

	/**
	 * Start the form import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   7.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'category_path':
						$this->setState('catid', $this->helper->getCategoryId($value));
						$this->setState($name, $value);
						break;
					case 'projecttitle':
						if ($value)
						{
							$this->setState('projectid', $this->helper->getProjectId($value, true));
							$this->setState($name, $value);
						}
						break;
					case 'state':
						switch (strtolower($value))
						{
							case 'n':
							case 'no':
							case '0':
								$value = 0;
								break;
							default:
								$value = 1;
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		// If no alias set use the title field
		$title = $this->getState('title', false);
		$alias = $this->getState('alias', false);

		if (!$alias && $title)
		{
			$alias = \JFilterOutput::stringURLSafe($title);
			$this->setState('alias', $alias);
		}

		if (!$title && !$alias)
		{
			$this->loaded = false;
			$this->log->addStats('skipped', \JText::_('COM_CSVI_NO_TITLE_ALIAS_FIELDS_FOUND'));
		}
		else
		{
			// Check if there is all required fields for import
			$requiredFields     = array('projectid~projecttitle', 'catid~category_path');
			$checkMissingFields = $this->fields->checkRequiredFields($requiredFields);

			// There must be an id or alias and catid or category_path
			if (!$checkMissingFields)
			{
				$this->loaded = true;

				if (!$this->getState('id', false))
				{
					$this->setState('id', $this->helper->getArticleId($this->getState('alias', '')));
				}

				// Load the current form2content data
				if ($this->f2carticle->load($this->getState('id', 0)))
				{
					if (!$this->template->get('overwrite_existing_data'))
					{
						$this->log->add('Article ' . $this->getState('alias') . 'not updated because the option overwrite existing data is set to No');
						$this->loaded = false;
					}
				}
			}
			else
			{
				$this->loaded = false;
				$this->log->addStats('skipped', $checkMissingFields);
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   7.0
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			if (!$this->getState('id', false) && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new products when user chooses to ignore new products
				$this->log->addStats('skipped', JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('alias', '')));
			}
			else
			{
				// Check if we have a title
				if (empty($this->f2carticle->title))
				{
					$this->f2carticle->title = $this->getState('alias');
				}

				// Set the attributes
				$this->setAttributes();

				// Check for meta data
				$this->setMetaData();

				// Check for extended
				$this->setExtended();

				// Get the transliteration helper
				$translit = new CsviHelperTranslit($this->template);

				if ($this->getState('alias', false) || ($this->getState('id', false) == 0))
				{
					$this->setState('alias', $translit->stringURLSafe($this->getState('alias')));
				}

				// Check if default intro and main template are set
				$this->getContentTypeTemplate();

				// Data must be in an array
				$data = ArrayHelper::fromObject($this->state);
				$contentData  = $data;

				if ($this->getState('id', false))
				{
					$this->contentTable->load($this->f2carticle->reference_id);
					$contentData['id'] = $this->f2carticle->reference_id;
				}

				// Add a creating date if there is no product_id
				if (!$this->getState('id', false))
				{
					$this->f2carticle->created = $this->date->toSql();
					$this->f2carticle->created_by = $this->userId;
					$this->f2carticle->publish_up = $this->date->toSql();

					$contentData['created'] = $this->date->toSql();
					$contentData['created_by'] = $this->userId;
				}

				$this->f2carticle->modified = $this->date->toSql();
				$contentData['modified'] = $this->date->toSql();

				if (!$this->getState('access', false))
				{
					$contentData['access'] = 1;
				}

				if (!$this->getState('language', false))
				{
					$contentData['language'] = '*';
				}

				$this->contentTable->save($contentData);
				$this->log->add('Joomla reference article added');

				if (!$this->getState('id', false))
				{
					$this->f2carticle->reference_id = $this->contentTable->id;
				}

				$this->f2carticle->bind($data);

				try
				{
					$this->f2carticle->check();
					$this->f2carticle->store();
					$this->formId = $this->f2carticle->id;

					if ($this->formId)
					{
						// Update content fields
						$this->processContentFields();
					}

					$this->clearTables();

					$this->log->add('F2C article added', false);
					$this->log->addStats('added', JText::_('COM_CSVI_TABLE_FORM2CONTENTTABLEF2CARTICLE_ADDED'));
				}
				catch (Exception $e)
				{
					$this->log->add('Cannot add article. Error: ' . $e->getMessage(), false);
					$this->log->addStats('incorrect', JText::_('COM_CSVI_TABLE_FORM2CONTENTTABLEF2CARTICLE_ERROR'));

					return false;
				}
			}

			return true;
		}

		return false;
	}

	/**
	 * Get a list of content fields that can be used as available field.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 *
	 * @throws  Exception
	 */
	private function loadContentFields()
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('fieldname'))
			->from($this->db->quoteName('#__f2c_projectfields'))
			->group($this->db->quoteName('fieldname'));
		$this->db->setQuery($query);
		$this->contentFields = $this->db->loadObjectList();

		$this->log->add('Load the content fields');
	}

	/**
	 * Get a default content type template.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 *
	 * @throws  Exception
	 */
	private function getContentTypeTemplate()
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('settings'))
			->from($this->db->quoteName('#__f2c_project'))
			->where($this->db->quoteName('id') . ' = ' . (int) $this->getState('projectid', false));
		$this->db->setQuery($query);
		$this->log->add('Load the content type templates');

		$settings = json_decode($this->db->loadResult());

		if (!$this->getState('intro_template', false))
		{
			$this->setState('intro_template', $settings->intro_template);
		}

		if (!$this->getState('main_template', false))
		{
			$this->setState('main_template', $settings->main_template);
		}
	}

	/**
	 * Update content fields data.
	 *
	 * @return  bool Returns true if all is OK | Returns false otherwise
	 *
	 * @since   7.0
	 */
	private function processContentFields()
	{
		if (count($this->contentFields) > 0)
		{
			foreach ($this->contentFields as $field)
			{
				$query = $this->db->getQuery(true);

				if ($this->getState($field->fieldname, ''))
				{
					$projectId = $this->getState('projectid', false);

					if ($projectId)
					{
						$fieldId   = $this->helper->getFieldId($field->fieldname, $projectId);

						if ($fieldId)
						{
							$query->delete($this->db->quoteName('#__f2c_fieldcontent'))
								->where($this->db->quoteName('formid') . '=' . (int) $this->formId)
								->where($this->db->quoteName('fieldid') . '=' . (int) $fieldId);
							$this->db->setQuery($query)->execute();
							$this->log->add('Delete content before inserting new values');

							$query->clear()
								->select($this->db->quoteName('name'))
								->from($this->db->quoteName('#__f2c_fieldtype', 'ft'))
								->leftJoin(
									$this->db->quoteName('#__f2c_projectfields', 'pf') . ' ON ' .
									$this->db->quoteName('pf.fieldtypeid') . ' = ' . $this->db->quoteName('ft.id')
								)
								->where($this->db->quoteName('pf.fieldname') . '=' . $this->db->quote($field->fieldname))
								->where($this->db->quoteName('pf.projectid') . '=' . (int) $projectId);
							$this->db->setQuery($query);
							$fieldType = $this->db->loadResult();

							$this->log->add('Process field type ' . $fieldType, false);

							switch (strtolower($fieldType))
							{
								case 'image':
									$finalImageArray = array();
									$imageArray = array(
										'filename',
										'widthThumbnail',
										'heightThumbnail',
										'width',
										'height',
										'alt',
										'title'
									);
									$imageValues    = $this->getState($field->fieldname, '');
									$fieldValue     = $imageValues;

									if (strpos($imageValues, '|') !== false)
									{
										$imageValuesArray = explode('|', $imageValues);

										foreach ($imageValuesArray as $imageValArray)
										{
											if (strpos($imageValArray, ':'))
											{
												list($imageName, $imageValue) = explode(':', $imageValArray);

												if (in_array($imageName, $imageArray))
												{
													$finalImageArray[$imageName] = $imageValue;
												}
												else
												{
													$this->log->add('Field ' . $imageValArray . ' is not an image field');
												}
											}
										}

										$fieldValue = json_encode($finalImageArray);
									}
									break;
								case 'displaylist':
									$fieldValue = $this->getState($field->fieldname, '');
									$this->log->add('Field value: ' . $fieldValue, false);

									if (strpos($fieldValue, '|') !== false)
									{
										$listArray  = explode('|', $fieldValue);
										$fieldValue = json_encode((object) $listArray);
									}
									break;
								default:
									$fieldValue = $this->getState($field->fieldname, '');
									break;
							}

							$columns = array(
								$this->db->quoteName('formid'),
								$this->db->quoteName('fieldid'),
								$this->db->quoteName('attribute'),
								$this->db->quoteName('content')
							);

							$values = array(
								(int) $this->formId,
								(int) $fieldId,
								$this->db->quote('VALUE'),
								$this->db->quote($fieldValue)
							);

							$query->clear()
								->insert($this->db->quoteName('#__f2c_fieldcontent'))
								->columns($columns)
								->values(implode(',', $values));

							try
							{
								$this->db->setQuery($query)->execute();
								$this->log->add('Insert the new fields content');
							}
							catch (Exception $e)
							{
								$this->log->addStats('incorrect', $e->getMessage());
								$this->log->add($e->getMessage(), false);
							}
						}
						else
						{
							$this->log->add('No field name found for ' . $field->fieldname, false);
						}
					}
					else
					{
						$this->log->add('No project ID found', false);
					}
				}
			}

			return true;
		}

		$this->log->add('No content fields found', false);

		return false;
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function loadTables()
	{
		$this->f2carticle   = $this->getTable('F2carticle');
		$this->contentTable = $this->getTable('Content');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function clearTables()
	{
		$this->f2carticle->reset();
		$this->contentTable->reset();
	}

	/**
	 * Set the attributes field.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	private function setAttributes()
	{
		// Check for attributes
		if (!$this->getState('attribs', false))
		{
			$attributeFields = array
			(
				'show_title',
				'link_titles',
				'show_intro',
				'show_category',
				'link_category',
				'show_parent_category',
				'link_parent_category',
				'show_author',
				'link_author',
				'show_create_date',
				'show_modify_date',
				'show_publish_date',
				'show_item_navigation',
				'show_icons',
				'show_print_icon',
				'show_email_icon',
				'show_vote',
				'show_hits',
				'show_noauth',
				'urls_position',
				'alternative_readmore',
				'article_layout',
				'show_publishing_options',
				'show_article_options',
				'show_urls_images_backend',
				'show_urls_images_frontend'
			);

			// Load the current attributes
			$attributes = json_decode($this->f2carticle->attribs);

			if (!is_object($attributes))
			{
				$attributes = new stdClass;
			}

			foreach ($attributeFields as $field)
			{
				$attributes->$field = $this->getState($field, '');

				if (!$this->getState($field, false))
				{
					if ($this->$field == '*')
					{
						$attributes->$field = '';
					}
					else
					{
						$attributes->$field = $this->getState($field, '');
					}
				}
			}

			// Store the new attributes
			$this->setState('attribs', json_encode($attributes));
		}
	}

	/**
	 * Set the meta data.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	private function setMetaData()
	{
		if (!$this->getState('metadata', false))
		{
			$metadataFields = array
			(
				'robots',
				'metaauthor',
				'rights',
				'xreference'
			);

			// Load the current images
			$metadata = json_decode($this->f2carticle->metadata);

			if (!is_object($metadata))
			{
				$metadata = new stdClass;
			}

			foreach ($metadataFields as $field)
			{
				$metadata->$field = $this->getState($field, '');

				if ($field === 'metaauthor')
				{
					$metadata->author = $this->getState('metaauthor', '');
				}
			}

			// Store the new metadata
			$this->setState('metadata', json_encode($metadata));
		}
	}

	/**
	 * Set extended options.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	private function setExtended()
	{
		if (!$this->getState('extended', false))
		{
			// Load the current settings
			$extended = json_decode($this->f2carticle->extended);

			if (!is_object($extended))
			{
				$extended = new stdClass;
			}

			$extended->tags = $this->getState('tags', '');

			// Store the new attributes
			$this->setState('extended', json_encode($extended));
		}
	}
}
